#!/usr/bin/python
# -*- coding: utf-8 -*-
"""場所打ち鋼管コンクリート杭の鋼管長の選定

『SS7』結果CSVから杭設計用応力と杭許容耐力を取得し、鋼管長を選定する。

"""


from Python import Ss7Python as Cmd
import csv


def import_csv(file_path: str) -> list:
    """CSVの読み込み

    指定したCSVファイルを読み込み、
    CSVデータをリストに格納する。

    Args:
        file_path (str): CSVファイルのパス
    Returns:
        list: CSVデータのリスト

    """

    data = []
    # CSVファイルを読み込む
    with open(file_path, mode="r", encoding="cp932") as file:
        reader = csv.reader(file)
        for row in reader:
            data.append(row)

    return data


def get_row_csv(data: list, target: str) -> list:
    """指定した文字列のCSVの行番号の取得

    指定した文字列のCSVファイルの行番号を取得する。

    Args:
        data (list): 文字列のリスト
        target (str): 検索したい文字列
    Returns:
        list: 行番号

    """

    line_no = []  # 行番号
    line_no = [i for i, line_s in enumerate(data) if target in line_s]
    return line_no


class CalcpileClass:
    """場所打ち鋼管コンクリート杭の計算クラス

    場所打ち鋼管コンクリート杭の計算クラスです。

    Attributes:
        m_ss7_version（str）:『SS7』のバージョン
        m_ss7_path（str）:『SS7』データのパス
        m_pilefugo（list）:杭符号 PileF
        m_pilekei（list）:杭径 PileD m
        m_ume（list）:埋込長 PileUme m
        m_depth（list）:深度 m
        m_stressm（list）:杭設計用応力 M kNm
        m_stressq（list）:杭設計用応力 Q kN
        m_strengthm（list）:杭許容耐力 Ma kNm
        m_strengthq（list）:杭許容耐力 Qa kN
        m_depthupm（list）:深度（上端）曲げ用 DUpm m
        m_depthdnm（list）:深度（下端） 曲げ用 DDnm m
        m_mdup（list）:設計曲げ（上端） MdUp kNm
        m_mddn（list）:設計曲げ（下端） MdDn kNm
        m_depthupq（list）:深度（上端） せん断用 DUpq m
        m_depthdnq（list）:深度（下端） せん断用 DDnq m
        m_qdup（list）:設計せん断（上端） QdUp kN
        m_qddn（list）:設計せん断（下端） QdDn kN
        m_lengthm（list）:曲げで決まる必要鋼管長 Lm m
        m_lengthq（list）:せん断で決まる必要鋼管長 Lq m
        m_minlength（list）:最小必要鋼管長 Lmin m
        m_needlength（list）:必要鋼管長 Lneed m

    """

    def __init__(self, ss7_version: str, ss7_path: str) -> None:
        self.m_ss7_version = ss7_version
        self.m_ss7_path = ss7_path
        self.m_bairitsu = 2.0  # 最小鋼管長の算定に用いる倍率

    def get_ss7datacsv_stiffness(self) -> None:
        """結果CSVデータの杭の剛性表の取得

        杭符号と杭径を取得する。

        """

        # 杭の剛性表の行番号
        row_stiffness = get_row_csv(self.m_csvdata, "name=杭の剛性表")
        # 杭符号の列番号
        col_pilefugo = get_row_csv(self.m_csvdata[row_stiffness[0] + 1], "杭符号")
        # 杭径の列番号
        col_pilekei = get_row_csv(self.m_csvdata[row_stiffness[0] + 1], "杭径")

        # 杭符号と杭径のセット
        self.m_pilefugo: list = []
        self.m_pilekei: list = []
        row = 0
        fugo = 0
        while True:
            if len(self.m_csvdata[row_stiffness[0] + 5 + row]) == 0:
                break
            if self.m_csvdata[row_stiffness[0] + 5 + row][0] != "":
                self.m_pilefugo.append(
                    str(self.m_csvdata[row_stiffness[0] + 5 + row][col_pilefugo[0]])
                )  # 杭符号
                self.m_pilekei.append(
                    float(self.m_csvdata[row_stiffness[0] + 5 + row][col_pilekei[0]])
                    * 0.001
                )  # 杭径 m
                fugo = fugo + 1
            row = row + 1

    def get_ss7datacsv_foundsect(self) -> None:
        """結果CSVデータの独立杭基礎断面リストの取得

        埋込長を取得する。

        """

        # 独立杭基礎断面リストの行番号
        row_foundsect = get_row_csv(self.m_csvdata, "name=独立杭基礎断面リスト")
        # 埋込長の列番号
        col_foundsect = get_row_csv(self.m_csvdata[row_foundsect[0] + 2], "埋込長")

        # 埋込長のセット
        self.m_ume: list = []
        end = len(self.m_pilefugo)
        for fugo in range(0, end, 1):
            row = 0
            while True:
                if len(self.m_csvdata[row_foundsect[0] + 6 + row]) == 0:
                    break
                if (
                    self.m_csvdata[row_foundsect[0] + 6 + row][1].split("-")[1]
                    == self.m_pilefugo[fugo].split()[0]
                ):
                    self.m_ume.append(
                        float(
                            self.m_csvdata[row_foundsect[0] + 6 + row][col_foundsect[0]]
                        )
                        * 0.001
                    )  # 埋込長 m
                    break
                row = row + 1

    def get_ss7datacsv_stress_sect(self) -> None:
        """結果CSVデータの杭の設計用応力表(詳細)と場所打ち杭断面算定表の取得

        杭設計用応力と杭許容耐力を取得する。

        """

        # 杭の設計用応力の行番号
        row_stress = get_row_csv(self.m_csvdata, "name=杭の設計用応力表(詳細)")
        # 杭の許容耐力の行番号
        row_strength = get_row_csv(self.m_csvdata, "name=場所打ち杭断面算定表")
        # 深度の列番号
        col_depth = get_row_csv(self.m_csvdata[row_stress[0] + 1], "深さ")
        # 杭の設計用応力 Mの列番号
        col_stressm = get_row_csv(self.m_csvdata[row_stress[0] + 1], "M")
        # 杭の設計用応力 Qの列番号
        col_stressq = get_row_csv(self.m_csvdata[row_stress[0] + 1], "Q")
        # 杭の許容耐力 Maの列番号
        col_strengthm = get_row_csv(self.m_csvdata[row_strength[0] + 2], "許容Ma")
        # 杭の許容耐力 Qaの列番号
        col_strengthq = get_row_csv(self.m_csvdata[row_strength[0] + 2], "許容Qa")

        # 杭の設計用応力のセット
        self.m_depth: list[list] = []
        self.m_stressm: list[list] = []
        self.m_stressq: list[list] = []
        row = 0
        end = len(self.m_pilefugo)
        for fugo in range(0, end, 1):
            self.m_depth.append([])
            self.m_stressm.append([])
            self.m_stressq.append([])
            while True:
                self.m_depth[fugo].append(
                    float(self.m_csvdata[row_stress[0] + 4 + row][col_depth[0]])
                )  # 深度 m
                self.m_stressm[fugo].append(
                    float(self.m_csvdata[row_stress[0] + 4 + row][col_stressm[0]])
                )  # 杭の設計用応力 M knm
                self.m_stressq[fugo].append(
                    float(self.m_csvdata[row_stress[0] + 4 + row][col_stressq[0]])
                )  # 杭の設計用応力 Q kn

                if self.m_csvdata[row_stress[0] + 4 + row][-1] == "<RE>":
                    row = row + 1
                    break
                row = row + 1

        # 杭の許容耐力のセット
        self.m_strengthm: list = []
        self.m_strengthq: list = []
        end = len(self.m_pilefugo)
        for fugo in range(0, end, 1):
            row = 0
            while True:
                if row_strength[0] + 5 + row == len(self.m_csvdata):
                    break
                if (
                    self.m_csvdata[row_strength[0] + 5 + row][0]
                    == self.m_pilefugo[fugo].split()[0]
                ):
                    self.m_strengthm.append(
                        float(
                            self.m_csvdata[row_strength[0] + 5 + row + 2][
                                col_strengthm[0]
                            ]
                        )
                    )  # 杭の許容耐力 Ma kNm
                    self.m_strengthq.append(
                        float(
                            self.m_csvdata[row_strength[0] + 5 + row + 2][
                                col_strengthq[0]
                            ]
                        )
                    )  # 杭の許容耐力 Qa kN
                    break
                row = row + 1

    def out_ss7datacsv(self) -> None:
        """結果CSVデータを出力し、データを取得

        『SS7』を起動して結果CSVを出力し、結果CSVからデータを取得する。

        """

        # 『SS7』結果CSVを出力する
        Cmd.Init()
        Cmd.Start(self.m_ss7_version, 1)  # 1=ログファイルをクリアする

        # データを開く
        data = Cmd.Open(self.m_ss7_path, 1, 3)  # 1=変換して開く 3=物件データは開かない

        # データが正しく開けたことの判定
        if data is not None:
            # 計算実行
            data.DeleteResult("結果1")  # 結果1がある場合削除
            data.Calculate("結果1", "断面算定")

            # エラー情報を取得する
            err = Cmd.GetLastError()

            # 計算が正しく実行できたことの判定
            if err.IsOK() is True:
                print("計算が成功しました")

                # 計算結果データを取得する
                res = data.GetResultData("結果1")

                # 結果CSVをエクスポートする
                res.ExportResultCsv(
                    "8.3.6.独立杭基礎;8.5.9.部材剛性（杭）;8.11.5.杭設計用応力表（詳細）;8.11.7.断面算定表（場所打ち杭）",
                    self.m_ss7_path + r"\output.csv",
                    1,
                    1,
                    2,
                )
                err = Cmd.GetLastError()

                # 結果CSVのエクスポートに失敗した場合はメッセージを出力し、プログラムを終了する
                if err.IsOK() is False:
                    print("結果CSVのエクスポートに失敗しました")
                    # データを閉じる
                    data.Close(2)  # 2=保存しない

                # 結果CSVを読み込む
                self.m_csvdata = import_csv(self.m_ss7_path + r"\output.csv")

                # 杭の剛性表の読み込み
                self.get_ss7datacsv_stiffness()

                # 独立杭基礎断面リストの読み込み
                self.get_ss7datacsv_foundsect()

                # 杭の設計用応力表(詳細)と場所打ち杭断面算定表の読み込み
                self.get_ss7datacsv_stress_sect()
            else:
                print("計算が失敗しました")
                # データを閉じる
                data.Close(2)  # 2=保存しない

        # データを閉じる
        data.Close(2)  # 2=保存しない
        Cmd.End(1)  # 1=保存して終了

    def cal_steellength(self) -> None:
        """場所打ち鋼管コンクリート杭の鋼管長の選定

        杭設計用応力と杭許容耐力の結果から、鋼管長を選定する。

        """
        self.m_depthupm: list = []  # 深度（上端） 曲げ用 m
        self.m_depthdnm: list = []  # 深度（下端） 曲げ用 m
        self.m_mdup: list = []  # 設計曲げ（上端） kNm
        self.m_mddn: list = []  # 設計曲げ（下端） kNm
        self.m_lengthm: list = []  # 曲げで決まる必要鋼管長 m
        self.m_depthupq: list = []  # 深度（上端） せん断用 m
        self.m_depthdnq: list = []  # 深度（下端） せん断用 m
        self.m_qdup: list = []  # 設計せん断（上端） kNm
        self.m_qddn: list = []  # 設計せん断（下端） kNm
        self.m_lengthq: list = []  # せん断で決まる必要鋼管長 m
        self.m_needlength: list = []  # 必要鋼管長 m
        self.m_minlength: list = []  # 最小鋼管長 m
        end = len(self.m_pilefugo)
        for fugo in range(0, end, 1):
            needlength = 0.0

            # 曲げで決まる必要鋼管長
            start = len(self.m_depth[fugo]) - 1
            for i in range(start, -1, -1):
                if self.m_stressm[fugo][i] > self.m_strengthm[fugo]:
                    self.m_depthupm.append(self.m_depth[fugo][i])
                    self.m_depthdnm.append(self.m_depth[fugo][i + 1])
                    self.m_mdup.append(self.m_stressm[fugo][i])
                    self.m_mddn.append(self.m_stressm[fugo][i + 1])
                    self.m_lengthm.append(
                        self.m_depth[fugo][i + 1]
                        - (self.m_depth[fugo][i + 1] - self.m_depth[fugo][i])
                        / (self.m_stressm[fugo][i] - self.m_stressm[fugo][i + 1])
                        * (self.m_strengthm[fugo] - self.m_stressm[fugo][i + 1])
                        + self.m_ume[fugo]
                    )
                    break
                if i == 0:
                    self.m_depthupm.append("-")
                    self.m_depthdnm.append("-")
                    self.m_mdup.append("-")
                    self.m_mddn.append("-")
                    self.m_lengthm.append(
                        self.m_bairitsu * self.m_pilekei[fugo] + self.m_ume[fugo]
                    )

            # せん断で決まる必要鋼管長
            start = len(self.m_depth[fugo]) - 1
            for i in range(start, -1, -1):
                if self.m_stressq[fugo][i] > self.m_strengthq[fugo]:
                    self.m_depthupq.append(self.m_depth[fugo][i])
                    self.m_depthdnq.append(self.m_depth[fugo][i + 1])
                    self.m_qdup.append(self.m_stressq[fugo][i])
                    self.m_qddn.append(self.m_stressq[fugo][i + 1])
                    self.m_lengthq.append(
                        self.m_depth[fugo][i + 1]
                        - (self.m_depth[fugo][i + 1] - self.m_depth[fugo][i])
                        / (self.m_stressq[fugo][i] - self.m_stressq[fugo][i + 1])
                        * (self.m_strengthq[fugo] - self.m_stressq[fugo][i + 1])
                        + self.m_ume[fugo]
                    )
                    break
                if i == 0:
                    self.m_depthupq.append("-")
                    self.m_depthdnq.append("-")
                    self.m_qdup.append("-")
                    self.m_qddn.append("-")
                    self.m_lengthq.append(
                        self.m_bairitsu * self.m_pilekei[fugo] + self.m_ume[fugo]
                    )

            # 必要鋼管長
            self.m_minlength.append(
                self.m_bairitsu * self.m_pilekei[fugo] + self.m_ume[fugo]
            )
            needlength = max(self.m_lengthm[fugo], self.m_lengthq[fugo])
            needlength = max(needlength, self.m_minlength[fugo])
            self.m_needlength.append(needlength)

    def out_steellength(self) -> None:
        """場所打ち鋼管コンクリート杭の選定した鋼管長の出力

        場所打ち鋼管コンクリート杭の選定した鋼管長を出力する。

        """
        # CSVファイルへ書き込む
        with open(self.m_ss7_path + r"\Output_NeedLength.csv", mode="w") as file:
            writer = csv.writer(file, lineterminator="\n")
            data = [
                [
                    "PileF",  # 杭符号
                    "PileD",  # 杭径 m
                    "PileUme",  # 埋込長 m
                    "DUpm",  # 深度（上端）曲げ用 m
                    "DDnm",  # 深度（下端）曲げ用 m
                    "MdUp",  # 設計曲げ（上端） kNm
                    "MdDn",  # 設計曲げ（下端） kNm
                    "Ma",  # 許容曲げ kNm
                    "Lm",  # 曲げで決まる必要鋼管長 m
                    "DUpq",  # 深度（上端）せん断用 m
                    "DDnq",  # 深度（下端）せん断用 m
                    "QdUp",  # 設計せん断（上端） kN
                    "QdDn",  # 設計せん断（下端） kN
                    "Qa",  # 許容せん断 kN
                    "Lq",  # せん断で決まる必要鋼管長 m
                    "Lmin",  # 最小鋼管長 m
                    "Lneed",  # 必要鋼管長 m
                ]
            ]
            writer.writerows(data)
            data = [
                [
                    "",  # 杭符号
                    "m",  # 杭径 m
                    "m",  # 埋込長 m
                    "m",  # 深度（上端）曲げ用 m
                    "m",  # 深度（下端）曲げ用 m
                    "kNm",  # 設計曲げ（上端） kNm
                    "kNm",  # 設計曲げ（下端） kNm
                    "kNm",  # 許容曲げ kNm
                    "m",  # 曲げで決まる必要鋼管長 m
                    "m",  # 深度（上端）せん断用 m
                    "m",  # 深度（下端）せん断用 m
                    "kN",  # 設計せん断（上端） kN
                    "kN",  # 設計せん断（下端） kN
                    "kN",  # 許容せん断 kN
                    "m",  # せん断で決まる必要鋼管長 m
                    "m",  # 最小鋼管長 m
                    "m",  # 必要鋼管長 m
                ]
            ]
            writer.writerows(data)
            end = len(self.m_pilefugo)
            for fugo in range(0, end, 1):
                data = [
                    [
                        self.m_pilefugo[fugo],  # 杭符号
                        self.m_pilekei[fugo],  # 杭径 m
                        self.m_ume[fugo],  # 埋込長 m
                        self.m_depthupm[fugo],  # 深度（上端）曲げ用 m
                        self.m_depthdnm[fugo],  # 深度（下端）曲げ用 m
                        self.m_mdup[fugo],  # 設計曲げ（上端） kNm
                        self.m_mddn[fugo],  # 設計曲げ（下端） kNm
                        self.m_strengthm[fugo],  # 許容曲げ kNm
                        self.m_lengthm[fugo],  # 曲げで決まる必要鋼管長 m
                        self.m_depthupq[fugo],  # 深度（上端）せん断用 m
                        self.m_depthdnq[fugo],  # 深度（下端）せん断用 m
                        self.m_qdup[fugo],  # 設計せん断（上端） kN
                        self.m_qddn[fugo],  # 設計せん断（下端） kN
                        self.m_strengthq[fugo],  # 許容せん断 kN
                        self.m_lengthq[fugo],  # せん断で決まる必要鋼管長 m
                        self.m_minlength[fugo],  # 最小鋼管長 m
                        self.m_needlength[fugo],  # 必要鋼管長 m
                    ]
                ]
                writer.writerows(data)


def main(ss7_version: str, ss7_path: str) -> None:
    """メイン処理

    最初に動く関数です。

    Args:
        ss7_version (str): 『SS7』のバージョン
        ss7_path (str): 『SS7』データのパス

    """

    # 場所打ち鋼管コンクリート杭の計算クラスのオブジェクトの作成
    calcpile = CalcpileClass(ss7_version, ss7_path)

    # 『SS7』結果CSVデータの取得
    calcpile.out_ss7datacsv()

    # 場所打ち鋼管コンクリート杭の鋼管長の選定
    calcpile.cal_steellength()

    # 場所打ち鋼管コンクリート杭の選定した鋼管長の出力
    calcpile.out_steellength()


if __name__ == "__main__":

    # 環境設定の指定

    # 『SS7』のバージョン
    ss7_version = "1.1.1.20"

    # 『SS7』データのパス
    ss7_path = r"C:\UsrData\Ss7Data\11806035.ikn"

    main(ss7_version, ss7_path)
